#if !defined INTERFACE_H
#define INTERFACE_H

#include <stdio.h>
// Node data
typedef struct
{
	float xyz[3]; //cartesian coordinates
	int id;		// node ID
	int ndf;	// number of degrees of freedom
	int config;	//node configuration
	int cid;	// coordinate frame for analysis results
	int pspc;	// constraint e.g. 2356 == 011011
	char gtype;	// node type
} NodeData;

// Element data
#define MNODES 30 // max number of element nodes
#define MADATA 20 // max number of additional data values	
typedef struct
{
	int id;		// element ID
	int shape;	// shape (1=point,2=bar,3=tri,4=quad,5=tet,7=wedge,8=hex)
	int n2;		// id of node in xy-plane
	int group;	// group ID
	int mechtype;	// mechType ID
	int nodes;	// number of element nodes in lnodes
	int pid;	// property ID
	int pid2;	// second property ID with config > 1000
	int nad;	// number of associated data values
	int lnodes[MNODES]; // element nodes ( == indix of nodedata + 1 !! )
	float adata[MADATA];	// additional data values
} ElementData;

// Material data
typedef struct
{
	int id;		// material ID
	int type;	// material type
	int n1;		// number of associated ply data values
	double data[100]; // 96 material property constants
	double adata[20];  // associated ply data
} MaterialProperties;

// Element properties
#define EPM 50
typedef struct
{
	int id;		// property ID
	int matid;	// material ID
	int config;	// configuration / type id
	int nv;		// number of data values
	int shape;	// element shape
	int nodes;	// number of element nodes
	float data[EPM];	// property data values
} ElementProperties;

// Coordinate frame
typedef struct
{
	int id;		// coordinate ID
	int ktyp;	// coordinate type (1=cartesian,2=cylindrical,3=spherical
	float coors[3][3];	// coordinates of 3 points (origin,on z-axis,on x-z-plane)
} CoorSystem;

// Distributed Loads
typedef struct
{
	int id;		// element id
	int loadset;	// load set id
	int ltyp;	// load type (0=line,1=surface)
	int nastrantyp;	// (10-14: NASTRAN: 10:PLOAD, 11:PLOA1, 12:PLOAD2, 14: PLOAD4
	int eflag;	// element flag (0 or 1)
	int gflag;	// node flag ( 0 or 1)
	int icomp;	// 6 load component flags e.g. 234 == 011100
	int node;	// 8 element node flags e.g. 1357 == 10101010
	int nfe;	// edge number (1-12) or face number (1-6)
	int ndw;	// number of data values
	float pdata[32]; // data values
	// data values NASTRAN: ltyp = 10:P,G1-G4 == indices of nodedata+1
	//		ltyp = 11: type,scale,x1,p1,x2,pe, 25 indices of elemdata+1
	//		ltyp = 12: P,30 indices of elemdata+1
	//		ltyp = 14: p1-p4,g1,g3,cid,n1-n3 (g1,g3 = indices of nodedata+1)
} DistributedLoads;

// Node forces
typedef struct
{
	int id;		// node id
	int loadset;	// load set id
	int cid;	// coordinate frame id
	int type;	// vector type (0:force,1:moment,2:displacement,3:rotation
	float scale;	// scale factor
	float fdata[3];	// non-zero force components
} NodeForces;

// all data values
typedef struct
{
	int ne;		// number of elements
	int np;		// number of nodes
	int nm;		// number of materials
	int nelp;	// number of element properties
	int nf;		// number of node forces	
	int nel;	// number od distributed loads
	int ncoor;	// number of coordinate frames
	NodeData *nodedata;
	ElementData *elemdata;
	MaterialProperties *mprop;
	ElementProperties *elemprop;
	CoorSystem *coorsys;
	DistributedLoads *elemloads;
	NodeForces *nodeforces;
} StructData;

// get all stored data in 1 data structure (function in interface.lib)
int getStructData( // rtv = 0 if successful / rtv = 1 if not successful
	char *project, // base file title
	StructData *structdata,	// pointer to data structure to store data in
	int global);	// transfer node forces to global koordinates

// free all memory allocated in getStructData (function in interface.lib)
void freeStructData(StructData *structdata);

int patran(FILE *pf,StructData *structdata,char *project);

#endif
